@@ -4,7 +4,7 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-from parlai.scripts.parlai_exe import Parlai as main
+from parlai.core.script import superscript_main as main
 
 if __name__ == '__main__':
     main()
