@@ -7,8 +7,9 @@
 """
 Integration with Hugging Face Transformers.
 
-Please see <https://huggingface.co/transformers/>. Currently, the only implementation is
-GPT2. To use this model, run with `-m hugging_face/gpt2`.
+Please see <https://huggingface.co/transformers/>. Currently, the only implementations
+are GPT2 and DialoGPT. To use these models, run with `-m hugging_face/gpt2` or `-m
+hugging_face/dialogpt`.
 """
 try:
     import transformers  # noqa: F401
@@ -20,5 +21,5 @@ class HuggingFaceAgent:
     def __init__(self, opt, shared=None):
         raise RuntimeError(
             '`-m hugging_face` is not a valid choice. Please run with '
-            '`-m hugging_face/gpt2`.'
+            '`-m hugging_face/gpt2` or `-m hugging_face/dialogpt`.'
         )
