@@ -12,6 +12,8 @@ from parlai.core.agents import Agent
 from parlai.core.dict import DictionaryAgent
 from parlai.utils.misc import maintain_dialog_history, load_cands
 from parlai.core.torch_agent import TorchAgent
+from parlai.utils.io import PathManager
+import parlai.utils.logging as logging
 from .modules import Starspace
 
 import torch
@@ -20,7 +22,6 @@ import torch.nn as nn
 from collections import deque
 
 import copy
-import os
 import random
 import json
 
@@ -189,17 +190,15 @@ class StarspaceAgent(Agent):
         self.ys_cache_sz = opt['cache_size']
         self.truncate = opt['truncate'] if opt['truncate'] > 0 else None
         self.history = {}
-        self.debugMode = False
         if shared:
-            torch.set_num_threads(1)
             # set up shared properties
             self.dict = shared['dict']
             self.model = shared['model']
         else:
-            print("[ creating StarspaceAgent ]")
+            logging.info("creating StarspaceAgent")
             # this is not a shared instance of this class, so do full init
             if opt.get('model_file') and (
-                os.path.isfile(opt.get('model_file') + '.dict')
+                PathManager.exists(opt.get('model_file') + '.dict')
                 or (opt['dict_file'] is None)
             ):
                 # set default dict-file if not set
@@ -208,7 +207,7 @@ class StarspaceAgent(Agent):
             self.dict = DictionaryAgent(opt)
 
             self.model = Starspace(opt, len(self.dict), self.dict)
-            if opt.get('model_file') and os.path.isfile(opt['model_file']):
+            if opt.get('model_file') and PathManager.exists(opt['model_file']):
                 self.load(opt['model_file'])
             else:
                 self._init_embeddings()
@@ -227,9 +226,9 @@ class StarspaceAgent(Agent):
             for c in self.fixedCands_txt:
                 fcs.append(torch.LongTensor(self.parse(c)).unsqueeze(0))
             self.fixedCands = fcs
-            print("[loaded candidates]")
+            logging.info("loaded candidates")
 
-    def _init_embeddings(self, log=True):
+    def _init_embeddings(self):
         """
         Copy embeddings from the pretrained embeddings to the lookuptable.
 
@@ -249,11 +248,10 @@ class StarspaceAgent(Agent):
                 )
                 weight.data[i] = vec
                 cnt += 1
-        if log:
-            print(
-                'Initialized embeddings for {} tokens ({}%) from {}.'
-                ''.format(cnt, round(cnt * 100 / len(self.dict), 1), name)
-            )
+        logging.info(
+            'Initialized embeddings for {} tokens ({}%) from {}.'
+            ''.format(cnt, round(cnt * 100 / len(self.dict), 1), name)
+        )
 
     def reset(self):
         """
@@ -289,9 +287,9 @@ class StarspaceAgent(Agent):
                 # skip non-model args
                 continue
             if k not in self.opt:
-                print('Adding new option [ {k}: {v} ]'.format(k=k, v=v))
+                logging.warning('Adding new option [ {k}: {v} ]'.format(k=k, v=v))
             elif self.opt[k] != v:
-                print(
+                logging.warning(
                     'Overriding option [ {k}: {old} => {v}]'.format(
                         k=k, old=self.opt[k], v=v
                     )
@@ -308,10 +306,6 @@ class StarspaceAgent(Agent):
             vec = [self.dict[self.dict.null_token]]
         return vec
 
-    def t2v(self, text):
-        p = self.dict.txt2vec(text)
-        return torch.LongTensor(p).unsqueeze(1)
-
     def v2t(self, vec):
         """
         Convert token indices to string of tokens.
@@ -363,25 +357,6 @@ class StarspaceAgent(Agent):
                 negs.append(query)
         return negs
 
-    def dict_neighbors(self, word, useRHS=False):
-        input = self.t2v(word)
-        W = self.model.encoder.lt.weight
-        q = W[input.data[0][0]]
-        if useRHS:
-            W = self.model.encoder2.lt.weight
-        score = torch.Tensor(W.size(0))
-        for i in range(W.size(0)):
-            score[i] = torch.nn.functional.cosine_similarity(q, W[i], dim=0).item()
-        val, ind = score.sort(descending=True)
-        for i in range(20):
-            print(
-                str(ind[i])
-                + " ["
-                + str(val[i])
-                + "]: "
-                + self.v2t(torch.Tensor([ind[i]]))
-            )
-
     def compute_metrics(self, loss, scores):
         metrics = {}
         pos = scores[0]
@@ -428,14 +403,7 @@ class StarspaceAgent(Agent):
                 if self.opt.get('input_dropout', 0) > 0:
                     xs, ys, negs = self.input_dropout(xs, ys, negs)
                 xe, ye = self.model(xs, ys, negs)
-                if self.debugMode:
-                    # print example
-                    print("inp: " + self.v2t(xs.squeeze()))
-                    print("pos: " + self.v2t(ys.squeeze()))
-                    for c in negs:
-                        print("neg: " + self.v2t(c.squeeze()))
-                    print("---")
-                y = -torch.ones(xe.size(0))
+                y = -(torch.ones(xe.size(0)))
                 y[0] = 1
                 loss = self.criterion(xe, ye, y)
                 loss.backward()
@@ -572,10 +540,6 @@ class StarspaceAgent(Agent):
         # call batch_act with this batch of one
         return self.batch_act([self.observation])[0]
 
-    def shutdown(self):
-        # """Save the state of the model when shutdown."""
-        super().shutdown()
-
     def save(self, path=None):
         """
         Save model parameters if model_file is set.
@@ -586,9 +550,9 @@ class StarspaceAgent(Agent):
             data['model'] = self.model.state_dict()
             data['optimizer'] = self.optimizer.state_dict()
             data['opt'] = self.opt
-            with open(path, 'wb') as handle:
+            with PathManager.open(path, 'wb') as handle:
                 torch.save(data, handle)
-            with open(path + '.opt', 'w') as handle:
+            with PathManager.open(path + '.opt', 'w') as handle:
                 json.dump(self.opt, handle)
 
     def load(self, path):
