@@ -13,13 +13,14 @@ from parlai.utils.bpe import bpe_factory, BPEHelper
 from .agents import Agent
 from .build_data import make_dir
 from collections import defaultdict
-import codecs
 import copy
 import numpy as np
 import os
 import json
 import re
 import parlai.utils.logging as logging
+from parlai.utils.io import PathManager
+from typing import List
 
 RETOK = re.compile(r'\w+|[^\w\s]|\n', re.UNICODE)
 
@@ -269,7 +270,7 @@ class DictionaryAgent(Agent):
             # If data built via pytorch data teacher, we need to load prebuilt dict
             if opt.get('dict_file'):
                 opt['dict_file'] = modelzoo_path(opt.get('datapath'), opt['dict_file'])
-                if os.path.isfile(opt['dict_file']):
+                if PathManager.exists(opt['dict_file']):
                     # load pre-existing dictionary
                     self.load(opt['dict_file'])
                     loaded = True
@@ -324,6 +325,38 @@ class DictionaryAgent(Agent):
             if opt.get('dict_file'):
                 self.save_path = opt['dict_file']
 
+    def add_additional_special_tokens(self, additional_special_tokens: List[str]):
+        """
+        Add additional special tokens to the dictionary.
+
+        Should only be called after initialization of the existing dictionary.
+        """
+        self.additional_special_tokens = additional_special_tokens
+
+        if (
+            self.additional_special_tokens
+            and not self.supports_additional_special_tokens()
+        ):
+            raise RuntimeError(
+                f'{self.tokenizer} does not currently support adding additional special tokens'
+            )
+
+        for tok in self.additional_special_tokens:
+            self.add_token(tok)
+
+        for i, tok in enumerate(self.additional_special_tokens):
+            self.freq[tok] = 1000000000 + 4 + i
+
+        if self.tokenizer == 'bytelevelbpe':
+            self.bpe.add_special_tokens(self, self.additional_special_tokens)
+
+    def supports_additional_special_tokens(self):
+        """
+        Indicates whether the dictionary supports additional special tokens.
+        """
+        # TODO: add to others
+        return self.tokenizer in ['bytelevelbpe', 'split', 'space']
+
     def is_prebuilt(self):
         """
         Indicates whether the dictionary is fixed, and does not require building.
@@ -569,7 +602,7 @@ class DictionaryAgent(Agent):
 
         lower_special = self.null_token == self.null_token.lower()
         SPECIAL_TOKENS = {'__UNK__', '__NULL__', '__END__', '__START__'}
-        with codecs.open(filename, 'r', encoding='utf-8', errors='ignore') as read:
+        with PathManager.open(filename, 'r', encoding='utf-8', errors='ignore') as read:
             for line in read:
                 split = line.strip().split('\t')
                 token = unescape(split[0])
@@ -612,14 +645,14 @@ class DictionaryAgent(Agent):
 
         make_dir(os.path.dirname(filename))
         mode = 'a' if append else 'w'
-        with open(filename, mode, encoding='utf-8') as write:
+        with PathManager.open(filename, mode, encoding='utf-8') as write:
             for i in self.ind2tok.keys():
                 tok = self.ind2tok[i]
                 cnt = self.freq[tok]
                 write.write('{tok}\t{cnt}\n'.format(tok=escape(tok), cnt=cnt))
 
         # save opt file
-        with open(filename + '.opt', 'w', encoding='utf-8') as handle:
+        with PathManager.open(filename + '.opt', 'w', encoding='utf-8') as handle:
             json.dump(self.opt, handle, indent=4)
         # save the byte level bpe model file as well
         if self.tokenizer == 'bytelevelbpe':
@@ -708,9 +741,13 @@ class DictionaryAgent(Agent):
             text = self.bpe.decode(tokens, vector, delimiter)
         elif self.tokenizer == 'bytelevelbpe':
             # We add special tokens in the beginning of ParlAI dict but in the
-            # end of Hugging Face dict,there is an offset of 4 between them.
+            # end of Hugging Face dict, there is an offset of #(extra tokens) between them.
+            extra_tokens = 4  # length of special tokens
             vector = [
-                idx + len(self.tok2ind) - 4 if idx < 4 else idx - 4 for idx in vector
+                self.bpe.special_tok_map[idx]
+                if idx in self.bpe.special_tok_map
+                else idx - extra_tokens
+                for idx in vector
             ]
             tokens = [self[int(idx)] for idx in vector]
             text = self.bpe.decode(tokens, vector, delimiter)
