@@ -7,18 +7,19 @@
 """
 Training script for ParlAI.
 
-The standard way to train a model. After training, also computes validation
-and test error.
+The standard way to train a model. After training, also computes
+validation and test error.
 
-The user must provide a model (with ``--model``) and a task (with ``--task``).
+The user must provide a model (with `--model`) and a task (with
+`--task`).
 
-Examples
---------
-.. code-block:: shell
+## Examples
 
-  python -m parlai.scripts.train_model -m ir_baseline -t dialog_babi:Task:1 -mf /tmp/model
-  python -m parlai.scripts.train_model -m seq2seq -t babi:Task10k:1 -mf '/tmp/model' -bs 32 -lr 0.5 -hs 128
-  python -m parlai.scripts.train_model -m drqa -t babi:Task10k:1 -mf /tmp/model -bs 10
+```shell
+parlai train_model -m ir_baseline -t dialog_babi:Task:1 -mf /tmp/model
+parlai train_model -m seq2seq -t babi:Task10k:1 -mf '/tmp/model' -bs 32 -lr 0.5 -hs 128
+parlai train_model -m drqa -t babi:Task10k:1 -mf /tmp/model -bs 10
+```
 """  # noqa: E501
 
 # TODO List:
@@ -26,7 +27,6 @@ Examples
 
 import json
 import numpy as np
-import os
 import signal
 from typing import Dict
 
@@ -46,8 +46,9 @@ from parlai.utils.distributed import (
     num_workers,
 )
 from parlai.utils.misc import Timer, nice_report
-from parlai.scripts.script import ParlaiScript
+from parlai.core.script import ParlaiScript, register_script
 import parlai.utils.logging as logging
+from parlai.utils.io import PathManager
 
 
 def setup_args(parser=None) -> ParlaiParser:
@@ -161,7 +162,7 @@ def setup_args(parser=None) -> ParlaiParser:
         '-lfc',
         '--load-from-checkpoint',
         type='bool',
-        default=False,
+        default=True,
         hidden=True,
         help='load model from checkpoint if available',
     )
@@ -258,7 +259,7 @@ class TrainLoop:
         if (
             opt['load_from_checkpoint']
             and opt.get('model_file')
-            and os.path.isfile(opt['model_file'] + '.checkpoint')
+            and PathManager.exists(opt['model_file'] + '.checkpoint')
         ):
             opt['init_model'] = opt['model_file'] + '.checkpoint'
             trainstats_suffix = '.checkpoint.trainstats'
@@ -276,6 +277,7 @@ class TrainLoop:
 
         # Create model and assign it to the specified task
         self.agent = create_agent(opt)
+        self.agent.opt.log()
         self.world = create_task(opt, self.agent)
         # set up timers
         self.train_time = Timer()
@@ -317,6 +319,7 @@ class TrainLoop:
 
         self.last_valid_epoch = 0
         self.valid_optim = 1 if opt['validation_metric_mode'] == 'max' else -1
+        self.train_reports = []
         self.valid_reports = []
         self.best_valid = None
 
@@ -327,26 +330,29 @@ class TrainLoop:
 
         # we may have been preempted, make sure we note that amount
         self._preempted_epochs = 0.0
-        if opt.get('model_file') and os.path.isfile(
+        if opt.get('model_file') and PathManager.exists(
             opt['model_file'] + trainstats_suffix
         ):
             # looks like we were preempted. make sure we load up our total
             # training stats, etc
-            with open(opt['model_file'] + trainstats_suffix) as ts:
+            with PathManager.open(opt['model_file'] + trainstats_suffix) as ts:
                 obj = json.load(ts)
                 self.parleys = obj.get('parleys', 0)
                 self._preempted_epochs = obj.get('total_epochs', 0)
                 self.train_time.total = obj.get('train_time', 0)
                 self.impatience = obj.get('impatience', 0)
                 self.valid_reports = obj.get('valid_reports', [])
+                self.train_reports = obj.get('train_reports', [])
                 if 'best_valid' in obj:
                     self.best_valid = obj['best_valid']
                 else:
                     # old method
-                    if opt.get('model_file') and os.path.isfile(
+                    if opt.get('model_file') and PathManager.exists(
                         opt['model_file'] + '.best_valid'
                     ):
-                        with open(opt['model_file'] + ".best_valid", 'r') as f:
+                        with PathManager.open(
+                            opt['model_file'] + ".best_valid", 'r'
+                        ) as f:
                             x = f.readline()
                             self.best_valid = float(x)
                             f.close()
@@ -386,17 +392,14 @@ class TrainLoop:
         if suffix:
             fn += suffix
         fn += '.trainstats'
-        with open(fn, 'w') as f:
+        with PathManager.open(fn, 'w') as f:
             json.dump(
                 {
                     'parleys': self.parleys,
                     'train_time': self.train_time.time(),
-                    'total_epochs': (
-                        self._preempted_epochs
-                        + num_workers() * self.world.get_total_epochs()
-                    ),
-                    'impatience': self.impatience,
-                    'valid_reports': [self._safe_report(v) for v in self.valid_reports],
+                    'total_epochs': self._total_epochs,
+                    'train_reports': self.train_reports,
+                    'valid_reports': self.valid_reports,
                     'best_valid': self.best_valid,
                 },
                 f,
@@ -420,8 +423,11 @@ class TrainLoop:
         valid_report = self._run_eval(
             self.valid_worlds, opt, 'valid', opt['validation_max_exs']
         )
-        v = valid_report.copy()
+        v = self._safe_report(valid_report.copy())
         v['train_time'] = self.train_time.time()
+        v['parleys'] = self.parleys
+        v['total_exs'] = self._total_exs
+        v['total_epochs'] = self._total_epochs
         self.valid_reports.append(v)
         # logging
         if opt['tensorboard_log'] and is_primary_worker():
@@ -551,9 +557,8 @@ class TrainLoop:
         # write to file
         if write_log and opt.get('model_file') and is_primary_worker():
             # Write out metrics
-            f = open(opt['model_file'] + '.' + datatype, 'a+')
-            f.write(f'{metrics}\n')
-            f.close()
+            with PathManager.open(opt['model_file'] + '.' + datatype, 'a+') as f:
+                f.write(f'{metrics}\n')
 
         return report
 
@@ -608,6 +613,13 @@ class TrainLoop:
         train_report = self._sync_metrics(train_report)
         self.world.reset_metrics()
 
+        train_report_trainstats = self._safe_report(train_report)
+        train_report_trainstats['total_epochs'] = self._total_epochs
+        train_report_trainstats['total_exs'] = self._total_exs
+        train_report_trainstats['parleys'] = self.parleys
+        train_report_trainstats['train_time'] = self.train_time.time()
+        self.train_reports.append(train_report_trainstats)
+
         # time elapsed
         logs.append(f'time:{self.train_time.time():.0f}s')
         logs.append(f'total_exs:{self._total_exs}')
@@ -641,11 +653,8 @@ class TrainLoop:
                 # do one example / batch of examples
                 try:
                     world.parley()
-                except StopTrainException:
-                    if is_distributed():
-                        raise RuntimeError(
-                            "StopTrainException not supported for " "distributed mode"
-                        )
+                except StopTrainException as e:
+                    logging.info(f"Stopping from {e}")
                     break
 
                 self.parleys += 1
@@ -688,10 +697,6 @@ class TrainLoop:
                         world.reset_metrics()
                         stop_training = self.validate()
                     except StopTrainException:
-                        if is_distributed():
-                            raise RuntimeError(
-                                "StopTrainException not supported for distributed mode"
-                            )
                         break
                     # reset the log time because we logged right before validating
                     self.log_time.reset()
@@ -749,6 +754,7 @@ class TrainLoop:
         return v_report, t_report
 
 
+@register_script('train_model', aliases=['tm', 'train'])
 class TrainModel(ParlaiScript):
     @classmethod
     def setup_args(cls):
@@ -756,8 +762,6 @@ class TrainModel(ParlaiScript):
 
     def run(self):
         self.train_loop = TrainLoop(self.opt)
-        self.parser.opt = self.train_loop.agent.opt
-        self.parser.print_args()
         return self.train_loop.train()
 
 
