@@ -11,20 +11,14 @@ import numpy as np
 from collections import Counter
 
 from parlai.core.agents import Agent
+from parlai.utils.io import PathManager
 from collections import defaultdict
 from parlai.core.teachers import FixedDialogTeacher
 from parlai.core.image_featurizers import ImageLoader
-from parlai.scripts.extract_image_feature import extract_feats
 from .build import build
 from parlai.tasks.coco_caption.build_2014 import buildImage as buildImage_2014
 from parlai.tasks.coco_caption.build_2015 import buildImage as buildImage_2015
 
-try:
-    import torch
-except ImportError:
-    raise ImportError('Need to install Pytorch: go to pytorch.org')
-from torch.utils.data import Dataset
-
 
 def _path(opt):
     build(opt)
@@ -138,7 +132,7 @@ class VqaDictionaryAgent(Agent):
                 self.tok2ind[self.unk_token] = index
                 self.ind2tok[index] = self.unk_token
 
-        if opt.get('dict_file') and os.path.isfile(opt['dict_file']):
+        if opt.get('dict_file') and PathManager.exists(opt['dict_file']):
             # load pre-existing dictionary
             self.load(opt['dict_file'])
 
@@ -281,7 +275,7 @@ class VqaDictionaryAgent(Agent):
         Initialize counts from other dictionary, or 0 if they aren't included.
         """
         print('Dictionary: loading dictionary from {}'.format(filename))
-        with open(filename) as read:
+        with PathManager.open(filename) as read:
             for line in read:
                 split = line.strip().split('\t')
                 token = unescape(split[0])
@@ -293,7 +287,7 @@ class VqaDictionaryAgent(Agent):
                     self.ind2tok[index] = token
         print('[ num ques words =  %d ]' % len(self.ind2tok))
 
-        with open(filename[:-5] + "_ans.dict") as read:
+        with PathManager.open(filename[:-5] + "_ans.dict") as read:
             for line in read:
                 split = line.strip().split('\t')
                 token = unescape(split[0])
@@ -332,13 +326,15 @@ class VqaDictionaryAgent(Agent):
         # if sort:
         #     self.sort()
 
-        with open(filename, 'a' if append else 'w') as write:
+        with PathManager.open(filename, 'a' if append else 'w') as write:
             for i in range(len(self.ind2tok)):
                 tok = self.ind2tok[i]
                 cnt = self.freq[tok]
                 write.write('{tok}\t{cnt}\n'.format(tok=escape(tok), cnt=cnt))
 
-        with open(filename[:-5] + "_ans.dict", 'a' if append else 'w') as write:
+        with PathManager.open(
+            filename[:-5] + "_ans.dict", 'a' if append else 'w'
+        ) as write:
             for i in range(len(self.ind2ans)):
                 tok = self.ind2ans[i]
                 cnt = self.ansfreq[tok]
@@ -352,127 +348,6 @@ class VqaDictionaryAgent(Agent):
             self.save(self.save_path)
 
 
-class VQADataset(Dataset):
-    """
-    A Pytorch Dataset utilizing streaming.
-    """
-
-    def __init__(self, opt):
-        self.opt = opt
-        self.use_att = opt.get('attention', False)
-        self.use_hdf5 = opt.get('use_hdf5', False)
-        self.opt['use_hdf5_extraction'] = self.use_hdf5
-        self.datatype = self.opt.get('datatype')
-        self.training = self.datatype.startswith('train')
-        self.num_epochs = self.opt.get('num_epochs', 0)
-        self.image_loader = ImageLoader(opt)
-        data_path, annotation_path, self.image_path = _path(opt)
-        self._setup_data(data_path, annotation_path, opt.get('unittest', False))
-        if self.use_hdf5:
-            try:
-                import h5py
-
-                self.h5py = h5py
-            except ImportError:
-                raise ImportError('Need to install h5py - `pip install h5py`')
-            self._setup_image_data()
-        self.dict_agent = VqaDictionaryAgent(opt)
-
-    def __getitem__(self, index):
-        index %= self.num_episodes()
-        qa = self.ques['questions'][index]
-        ep = {
-            'text': qa['question'],
-            'image': self.get_image(qa['image_id']),
-            'episode_done': True,
-        }
-        if self.opt.get('extract_image', False):
-            ep['image_id'] = qa['image_id']
-            return ep
-        if not self.datatype.startswith('test'):
-            anno = self.annotation['annotations'][index]
-            labels = [ans['answer'] for ans in anno['answers']]
-            ep['labels'] = [ans['answer'] for ans in anno['answers']]
-            ep['valid'] = True
-            if 'mc_label' in ep:
-                if not ep['mc_label'][0] in self.dict_agent.ans2ind:
-                    ep['valid'] = False
-            ep = self.dict_agent.encode_question([ep], self.training)
-            ep = self.dict_agent.encode_answer(ep)
-            ep[0]['labels'] = labels
-        else:
-            ep['valid'] = True
-            ep = self.dict_agent.encode_question([ep], False)
-        ep[0]['use_att'] = self.use_att
-        ep[0]['use_hdf5'] = self.use_hdf5
-        return (index, ep)
-
-    def __len__(self):
-        num_epochs = self.num_epochs if self.num_epochs > 0 else 100
-        num_iters = num_epochs if self.training else 1
-        return int(num_iters * self.num_episodes())
-
-    def _load_lens(self):
-        with open(self.length_datafile) as length:
-            lengths = json.load(length)
-            self.num_eps = lengths['num_eps']
-            self.num_exs = lengths['num_exs']
-
-    def _setup_data(self, data_path, annotation_path, unittest):
-        with open(data_path) as data_file:
-            self.ques = json.load(data_file)
-        if not self.datatype.startswith('test'):
-            with open(annotation_path) as data_file:
-                self.annotation = json.load(data_file)
-        if unittest:
-            self.ques['questions'] = self.ques['questions'][:10]
-            if not self.datatype.startswith('test'):
-                self.annotation['annotations'] = self.annotation['annotations'][:10]
-        self.image_paths = set()
-        for qa in self.ques['questions']:
-            self.image_paths.add(self.image_path + '%012d.jpg' % (qa['image_id']))
-
-    def _setup_image_data(self):
-        """
-        hdf5 image dataset.
-        """
-        extract_feats(self.opt)
-        im = self.opt.get('image_mode')
-        if self.opt.get('attention', False):
-            hdf5_path = self.image_path + 'mode_{}.hdf5'.format(im)
-        else:
-            hdf5_path = self.image_path + 'mode_{}_noatt.hdf5'.format(im)
-        hdf5_file = self.h5py.File(hdf5_path, 'r')
-        self.image_dataset = hdf5_file['images']
-
-        image_id_to_idx_path = self.image_path + 'mode_{}_id_to_idx.txt'.format(im)
-        with open(image_id_to_idx_path, 'r') as f:
-            self.image_id_to_idx = json.load(f)
-
-    def get_image(self, image_id):
-        if not self.use_hdf5:
-            im_path = self.image_path + '%012d.jpg' % (image_id)
-            return self.image_loader.load(im_path)
-        else:
-            img_idx = self.image_id_to_idx[str(image_id)]
-            return torch.Tensor(self.image_dataset[img_idx])
-
-    def num_episodes(self):
-        return len(self.ques['questions'])
-
-    def num_examples(self):
-        return self.num_episodes()
-
-    def num_images(self):
-        if not hasattr(self, 'num_imgs'):
-            self.num_imgs = len({q['image_id'] for q in self.ques['questions']})
-        return self.num_imgs
-
-
-class DefaultDataset(VQADataset):
-    pass
-
-
 class OeTeacher(FixedDialogTeacher):
     """
     VQA Open-Ended teacher, which loads the json vqa data and implements its own `act`
@@ -564,12 +439,12 @@ class OeTeacher(FixedDialogTeacher):
 
     def _setup_data(self, data_path, annotation_path):
         print('loading: ' + data_path)
-        with open(data_path) as data_file:
+        with PathManager.open(data_path) as data_file:
             self.ques = json.load(data_file)
 
         if not self.datatype.startswith('test'):
             print('loading: ' + annotation_path)
-            with open(annotation_path) as data_file:
+            with PathManager.open(annotation_path) as data_file:
                 self.annotation = json.load(data_file)
 
 
