@@ -21,6 +21,7 @@ from parlai.core.build_data import download, make_dir
 from parlai.core.opt import Opt
 from parlai.utils.misc import warn_once
 from parlai.utils.typing import TShared
+from parlai.utils.io import PathManager
 import parlai.utils.logging as logging
 
 try:
@@ -122,6 +123,13 @@ class BPEHelper(ABC):
             hidden=True,
             help='add prefix space before encoding',
         )
+        parser.add_argument(
+            '--hf-skip-special-tokens',
+            hidden=True,
+            type='bool',
+            default=True,
+            help='do not decode special tokens with bytelevelbpe',
+        )
         return parser
 
     @final
@@ -282,17 +290,14 @@ class SubwordBPEHelper(BPEHelper):
         """
         super().__init__(opt, shared)
         if not SUBWORD_BPE_INSTALLED:
-            raise RuntimeError(
-                "Please run \"pip install 'git+https://github.com/rsennrich"
-                "/subword-nmt.git#egg=subword-nmt'\""
-            )
+            raise RuntimeError("Please run `pip install subword-nmt`")
         if not opt.get('dict_file'):
             raise RuntimeError('--dict-file is mandatory.')
 
         self.splitter = re.compile(r'\w+|[^\w\s]', re.UNICODE)
 
         self.codecs = f"{opt['dict_file']}.codecs"
-        if os.path.exists(self.codecs):
+        if PathManager.exists(self.codecs):
             self._load_from_codecs()
 
     def helper_encode(self, text: str) -> List[str]:
@@ -370,8 +375,8 @@ class SubwordBPEHelper(BPEHelper):
             minfreq = 2
 
         codec_dir, _ = os.path.split(self.codecs)
-        os.makedirs(codec_dir, exist_ok=True)
-        with open(self.codecs, 'w', encoding='utf-8') as outstream:
+        PathManager.mkdirs(codec_dir)
+        with PathManager.open(self.codecs, 'w', encoding='utf-8') as outstream:
             learn_bpe.learn_bpe(
                 dictionary,
                 outstream,
@@ -387,7 +392,7 @@ class SubwordBPEHelper(BPEHelper):
         """
         Load BPE from codecs file.
         """
-        with open(self.codecs, 'r', encoding='utf-8') as codecs_file:
+        with PathManager.open(self.codecs, 'r', encoding='utf-8') as codecs_file:
             self.bpe = apply_bpe.BPE(codecs_file)
 
     def copy_codecs_file(self, target_file: str):
@@ -397,8 +402,8 @@ class SubwordBPEHelper(BPEHelper):
         :param target_file:
             where to copy the codecs.
         """
-        with open(target_file, 'w', encoding='utf-8') as wfile:
-            with open(self.codecs, encoding='utf-8') as rfile:
+        with PathManager.open(target_file, 'w', encoding='utf-8') as wfile:
+            with PathManager.open(self.codecs, encoding='utf-8') as rfile:
                 for line in rfile:
                     wfile.write(line)
 
@@ -458,14 +463,14 @@ class Gpt2BpeHelper(BPEHelper):
                 ' (no --dict-minfreq or --dict-maxtokens).'
             )
 
-        bpe_data, json_path = self._build_data()
+        self.bpe_data, self.json_path, self.merge_path = self._build_data()
 
         # build encoder & decoder
-        self.encoder: Dict[str, str] = self._build_encoder(json_path)
+        self.encoder: Dict[str, str] = self._build_encoder(self.json_path)
         self.decoder: Dict[str, str] = {v: k for k, v in self.encoder.items()}
 
         bpe_merges = [
-            tuple(merge_str.split()) for merge_str in bpe_data.split('\n')[1:-1]
+            tuple(merge_str.split()) for merge_str in self.bpe_data.split('\n')[1:-1]
         ]
         self.byte_encoder = self.bytes_to_unicode()
         self.byte_decoder = {v: k for k, v in self.byte_encoder.items()}
@@ -495,14 +500,14 @@ class Gpt2BpeHelper(BPEHelper):
         data_path = os.path.join(self.opt['datapath'], 'gpt2')
         vocab_path = os.path.join(data_path, 'vocab.bpe')
         json_path = os.path.join(data_path, 'encoder.json')
-        if not os.path.isfile(vocab_path) or not os.path.isfile(json_path):
+        if not PathManager.exists(vocab_path) or not PathManager.exists(json_path):
             make_dir(data_path)
             download(self.DEFAULT_VOCAB_BPE, data_path, 'vocab.bpe')
             download(self.DEFAULT_ENCODER_JSON, data_path, 'encoder.json')
-        with open(vocab_path, 'r', encoding="utf-8") as f:
+        with PathManager.open(vocab_path, 'r', encoding="utf-8") as f:
             bpe_data = f.read()
 
-        return bpe_data, json_path
+        return bpe_data, json_path, vocab_path
 
     def _build_encoder(self, json_path: str) -> Dict[str, str]:
         """
@@ -514,7 +519,7 @@ class Gpt2BpeHelper(BPEHelper):
         :return:
             encoder, mapping tokens to unicode reps
         """
-        with open(json_path, 'r', encoding='utf8') as f:
+        with PathManager.open(json_path, 'r', encoding='utf8') as f:
             encoder = json.load(f)
         for each_token in encoder.keys():
             new_token = ''.join(
@@ -673,6 +678,26 @@ class Gpt2BpeHelper(BPEHelper):
             dict_agent.add_token(each_token)
             dict_agent.freq[each_token] = 1
 
+    def save(self, dir_name: str, file_name: str):
+        """
+        Save appropriate files.
+
+        :param dir_name:
+            directory to save.
+        :param file_name:
+            file to save.
+        """
+        out_json_path = os.path.join(dir_name, file_name + "-vocab.json")
+        out_merge_path = os.path.join(dir_name, file_name + "-merges.txt")
+        # Possibly bad assumption: if the destination file already exists,
+        # we don't need to copy it over again.
+        if not PathManager.exists(out_json_path):
+            logging.info(f"Copying {self.json_path} to {out_json_path}")
+            PathManager.copy(self.json_path, out_json_path)
+        if not PathManager.exists(out_merge_path):
+            logging.info(f"Copying {self.merge_path} to {out_merge_path}")
+            PathManager.copy(self.merge_path, out_merge_path)
+
 
 ###################
 # HuggingFace BPE #
@@ -689,14 +714,16 @@ class HuggingFaceBpeHelper(BPEHelper):
     def __init__(self, opt: Opt, shared: TShared = None):
         super().__init__(opt, shared)
         # Default true for HF
+        self.special_tok_map = {}  # map from HF
         self.add_prefix_space = opt.get('bpe_add_prefix_space', True)
+        self.skip_special_tokens = opt.get('hf_skip_special_tokens', True)
         if self.add_prefix_space is None:
             self.add_prefix_space = True
         if opt.get('dict_loaded'):
             dfname = opt['dict_file']
-            if os.path.isfile(f'{dfname}-merges.txt'):
+            if PathManager.exists(f'{dfname}-merges.txt'):
                 opt['bpe_merge'] = f'{dfname}-merges.txt'
-            if os.path.isfile(f'{dfname}-vocab.json'):
+            if PathManager.exists(f'{dfname}-vocab.json'):
                 opt['bpe_vocab'] = f'{dfname}-vocab.json'
         try:
             from tokenizers import ByteLevelBPETokenizer
@@ -726,11 +753,11 @@ class HuggingFaceBpeHelper(BPEHelper):
                 '--dict-tokenizer bytelevelbpe'
             )
 
-        if not os.path.isfile(self.vocab_path):
+        if not PathManager.exists(self.vocab_path):
             raise IOError(
                 f'File {self.vocab_path} does not exist. --bpe-vocab must be pretrained.'
             )
-        if not os.path.isfile(self.merge_path):
+        if not PathManager.exists(self.merge_path):
             raise IOError(
                 f'File {self.merge_path} does not exist. --bpe-merge must be pretrained.'
             )
@@ -769,9 +796,24 @@ class HuggingFaceBpeHelper(BPEHelper):
         :return text:
             decoded text
         """
-        text = self.tokenizer.decode(token_ids)
+        text = self.tokenizer.decode(
+            token_ids, skip_special_tokens=self.skip_special_tokens
+        )
+
         return text
 
+    def add_special_tokens(self, dict_agent, special_tokens: List[str]):
+        """
+        Add special tokens to the tokenizer and dict_agent.
+        """
+        logging.debug(f'adding the following special tokens: {special_tokens}')
+        self.tokenizer.add_special_tokens(special_tokens)  # add to HF
+
+        for tok in special_tokens:
+            parlai_key = dict_agent[tok]
+            hf_key = self.tokenizer.token_to_id(tok)
+            self.special_tok_map[parlai_key] = hf_key
+
     def sync_with_dict(self, dict_agent):
         """
         Sync the dictionary agent with Hugging Face tokenizer's BPE dict.
@@ -784,8 +826,9 @@ class HuggingFaceBpeHelper(BPEHelper):
             dict_agent.end_token,
             dict_agent.unk_token,
         ]
-        self.tokenizer.add_special_tokens(special_tokens)
-        for i in range(self.tokenizer.get_vocab_size() - 4):
+        self.add_special_tokens(dict_agent, special_tokens)
+
+        for i in range(self.tokenizer.get_vocab_size() - len(special_tokens)):
             token = self.tokenizer.id_to_token(i)
             dict_agent.add_token(token)
             # We don't have access to the hugging face word frequency table,
@@ -801,7 +844,7 @@ class HuggingFaceBpeHelper(BPEHelper):
         :param file_name:
             file to save.
         """
-        self.tokenizer.save(dir_name, file_name)
+        self.tokenizer.save_model(dir_name, file_name)
 
 
 class SlowBytelevelBPE(Gpt2BpeHelper):
@@ -824,18 +867,18 @@ class SlowBytelevelBPE(Gpt2BpeHelper):
         vocab_path = ''
         if self.opt.get('dict_loaded'):
             dfname = self.opt['dict_file']
-            if os.path.isfile(f'{dfname}-merges.txt'):
+            if PathManager.exists(f'{dfname}-merges.txt'):
                 vocab_path = f'{dfname}-merges.txt'
-            if os.path.isfile(f'{dfname}-vocab.json'):
+            if PathManager.exists(f'{dfname}-vocab.json'):
                 json_path = f'{dfname}-vocab.json'
 
-        if os.path.isfile(vocab_path) and os.path.isfile(json_path):
-            with open(vocab_path, 'r', encoding="utf-8") as f:
+        if PathManager.exists(vocab_path) and PathManager.exists(json_path):
+            with PathManager.open(vocab_path, 'r', encoding="utf-8") as f:
                 bpe_data = f.read()
         else:
-            bpe_data, json_path = super()._build_data()
+            return super()._build_data()
 
-        return bpe_data, json_path
+        return bpe_data, json_path, vocab_path
 
     def sync_with_dict(self, dict_agent):
         """
