@@ -10,6 +10,7 @@ File for miscellaneous utility functions and constants.
 from collections import deque, OrderedDict
 from typing import Union, Optional, Set, Any, Dict, List, Tuple
 from datetime import timedelta
+import functools
 import math
 import time
 import re
@@ -18,6 +19,7 @@ import json
 
 from parlai.core.message import Message
 from parlai.utils.strings import colorize
+from parlai.utils.io import PathManager
 import parlai.utils.logging as logging
 
 try:
@@ -127,7 +129,7 @@ def load_cands(path, lines_have_ids=False, cands_are_replies=False):
         return None
     cands = []
     cnt = 0
-    with open(path) as read:
+    with PathManager.open(path) as read:
         for line in read:
             line = line.strip().replace('\\n', '\n')
             if len(line) > 0:
@@ -247,7 +249,8 @@ class TimeLogger:
             done = done.value()
         self.tot_time += self.timer.time()
         self.timer.reset()
-        report['exs'] = done
+        if report:
+            report['exs'] = done
         if total > 0 and done > 0:
             progress = done / total
             seconds_left = max(0, self.tot_time / progress - self.tot_time)
@@ -407,12 +410,17 @@ def nice_report(report) -> str:
         df = pd.DataFrame([output])
         df.columns = pd.MultiIndex.from_tuples(df.columns)
         df = df.stack().transpose().droplevel(0, axis=1)
-        result = "   " + df.to_string(
-            na_rep="",
-            line_width=line_width - 3,  # -3 for the extra spaces we add
-            float_format=float_formatter,
-            index=df.shape[0] > 1,
-        ).replace("\n\n", "\n").replace("\n", "\n   ")
+        result = (
+            "   "
+            + df.to_string(
+                na_rep="",
+                line_width=line_width - 3,  # -3 for the extra spaces we add
+                float_format=float_formatter,
+                index=df.shape[0] > 1,
+            )
+            .replace("\n\n", "\n")
+            .replace("\n", "\n   ")
+        )
         result = re.sub(r"\s+$", "", result)
         return result
     else:
@@ -442,7 +450,7 @@ def round_sigfigs(x: Union[float, 'torch.Tensor'], sigfigs=4) -> float:
     try:
         if x_ == 0:
             return 0
-        return round(x_, -math.floor(math.log10(abs(x_)) - sigfigs + 1))
+        return round(x_, -(math.floor(math.log10(abs(x_)) - sigfigs + 1)))
     except (ValueError, OverflowError) as ex:
         if x_ in [float('inf'), float('-inf')] or x_ != x_:  # inf or nan
             return x_
@@ -751,3 +759,14 @@ def error_once(msg: str) -> None:
     if msg not in _seen_logs:
         _seen_logs.add(msg)
         logging.error(msg)
+
+
+def recursive_getattr(obj, attr, *args):
+    """
+    Recursive call to getattr for nested attributes.
+    """
+
+    def _getattr(obj, attr):
+        return getattr(obj, attr, *args)
+
+    return functools.reduce(_getattr, [obj] + attr.split('.'))
