@@ -14,5 +14,5 @@ if __name__ == '__main__':
     parser.set_params(
         task='squad:fulldoc', model='drqa', model_file='models:drqa/squad/model'
     )
-    opt = parser.parse_args(print_args=False)
-    eval_model(opt, print_parser=parser)
+    opt = parser.parse_args()
+    eval_model(opt)
