@@ -9,5 +9,5 @@ from parlai.tasks.self_feeding.build import build
 
 
 if __name__ == '__main__':
-    opt = params.ParlaiParser().parse_args(print_args=False)
+    opt = params.ParlaiParser().parse_args()
     build(opt)
